package cn.edu.zjvtit.cloudstorage.apiservice.controller;

import cn.edu.zjvtit.cloudstorage.apiservice.ApiServiceApplication;
import cn.edu.zjvtit.cloudstorage.apiservice.rabbitmq.topic.Sender;
import cn.edu.zjvtit.cloudstorage.apiservice.store.DataServerStore;
import com.rabbitmq.client.Channel;
import org.slf4j.Logger;
import org.springframework.amqp.core.BindingBuilder;
import org.springframework.amqp.core.Message;
import org.springframework.amqp.core.Queue;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import cn.edu.zjvtit.cloudstorage.apiservice.result.Result;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.file.Files;
import java.util.Map;
import java.util.logging.Level;

@RestController
public class ObjectsController {
    @Autowired
    private DataServerStore dataServerStore;
    @Autowired
    private Sender sender;

    @Autowired
    private Logger logger;
    @Autowired
    private RestTemplate restTemplate;
    @Autowired
    private RabbitTemplate rabbitTemplate;

    @PutMapping("/objects/{filename}")
    public String putObject(@RequestBody byte[] requestBody, @RequestHeader Map<String, String> headers, @PathVariable("filename") String filename) throws IOException {
        String server = this.dataServerStore.selectRandomServer();
        //获取一个节点服务
        logger.info("put object to server: " + server);
        //日志
        if (server == null) {
            return new Result("no server available", "10000").toString();
        }
        //20秒没有服务节点返回输出 no server available
        String url = "http://" + server + "/objects/" + filename;
        // 发送post请求
        HttpEntity<byte[]> request = new HttpEntity<>(requestBody);
        this.restTemplate.exchange(url, HttpMethod.PUT, request, String.class);
        return (new Result("success", "10000")).toString();
    }

    @GetMapping("/objects/{filename}")
    public String getObject(HttpServletResponse response, @PathVariable("filename") String filename) throws IOException {
        // 查询哪台服务器有这个文件
        // 读取文件
        String queueName = filename;
        Channel channel = rabbitTemplate.getConnectionFactory().createConnection().createChannel(false);
        //获取连接
        channel.queueDeclare(queueName, false, false, false, null);
        //申明一个消息队列
        channel.queueBind(queueName, "exchange", "");
        //交换机和消息队列绑定
        sender.send(queueName);
        try {
            Object msg = rabbitTemplate.receiveAndConvert(queueName, 200);
            assert msg != null;
            //确定消息msg是不是为空
            logger.info(msg.toString());
            //输出消息日志
            String url = "http://" + msg + "/objects/" + filename;
            //用字符串拼接获取url地址
            ResponseEntity<String> res = this.restTemplate.getForEntity(url,String.class);
            response.reset();
            //重置响应
            response.setContentType("application/octet-stream");
            //设置响应类型
            response.setCharacterEncoding("utf-8");
            //设置编码格式
            response.setContentLength((int) res.getHeaders().getContentLength());
            //设置内容类型长度
            response.setHeader("Content-Disposition", "attachment;filename=" + filename );
            //设置响应头
            OutputStream os = response.getOutputStream()
            //将输出的文本保存
            os.write(res.getBody().getBytes());
            //输出文本
            os.close();
            //关闭书写程序释放内存
            return "";
        } catch (Exception e) {
            logger.info("locate object error", e);
            return new Result("file not exists", "10000").toString();
            //若出现异常跳到catch 1.输出消息日志错误 2.等待10秒返回文件不存在
        }
    }

    @GetMapping("/locate/{filename}")
    public String locateObject(@PathVariable("filename") String filename) throws IOException {
        // 查询哪台服务器有这个文件
        String queueName = filename;
        Channel channel = rabbitTemplate.getConnectionFactory().createConnection().createChannel(false);
        //获取连接
        channel.queueDeclare(queueName, false, false, false, null);
        //申明一个消息队列
        channel.queueBind(queueName, "exchange", "");

        sender.send(queueName);
        //发送文件
        try {
            Object msg = rabbitTemplate.receiveAndConvert(queueName, 200);
            assert msg != null;
            logger.info(msg.toString());
            return new Result("file in server:" + msg.toString(), "10000").toString();
        } catch (Exception e) {
            logger.info("locate object error", e);
            return new Result("file not exists", "10000").toString();
        }

    }


}
